<?php

namespace Teknisa\PLA\Modules\Requisicao\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Libs\Util\DateUtil;


class Pla00900RequisicaoExtra extends \Teknisa\GER\Controllers\TeknisaCrud {
	protected $crudService;
	protected $dataSourceName;
	protected $parameterBagColumns = array();
	protected $environment;
	protected $connection;
	protected $newCode;
	protected $qr2Strategy;
	protected $dataSourceManager;
	protected $service;
	protected $dataSourceToSaveProdPrato = "pla00900_saveTipoPrato_Prod";
	protected $dateUtil;
	protected $libUtilities;

	public function __construct($crudService, $parameterBagColumns, $environment, $connection, $newCode, $dataSourceManager, $service, $qr2Strategy, $entityManager, $dateUtil,$libUtilities) {
		$this->crudService = $crudService;
		$this->parameterBagColumns = $parameterBagColumns;
		$this->environment = $environment;
		$this->connection = $connection;
		$this->qr2Strategy = $qr2Strategy;
		$this->newCode = $newCode;
		$this->dataSourceManager = $dataSourceManager;
		$this->service = $service;
		$this->entityManager = $entityManager;
		$this->dateUtil = $dateUtil;
		$this->libUtilities  = $libUtilities;
		
	}
	
	protected $reportMapping = array (
    	'pla05700' => array (
            'strategy' => 'QR2',
            'parameterMapping' => 
            array (
                'CDFILIAL'       => 'P_CDFILIAL',
                'DTINICIAL'      => 'P_DTINICIAL',
                'DTFINAL'        => 'P_DTFINAL',
                'CDSERVFILI'     => 'P_CDSERVFILI',
                'TAMNIV'		 => 'P_TAMNIV',
                'NRLINHABRANCA'  => 'P_NRLINHABRANCA'
            )
        )
    );

	public function getFilial(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('NRORG');
		$this->dataSourceName = 'pla00900_filial';
		parent::find($request, $response);
	}
	
	public function paramGeral(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('NRORG');
		$this->dataSourceName = 'pla00900_paramgeral';
		parent::find($request, $response);
	}
	
	public function checkOperator(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDOPERADOR', 'P_CDGRUPOPER', 'NRORG');
		$this->dataSourceName = 'pla00900_auth_operator';
		parent::find($request, $response);
	}
	
	public function authOperator(DTO\Request $request, DTO\Response $response) {
		try{

			$row = $request->getRow();

			$dataSourceParamgeral = 'pla00900_paramgeral';
			$cdGrupOperator = $this->crudService->getFilteredDataSet($dataSourceParamgeral, []);
			$cdGrupOperator = $cdGrupOperator->getRows()[0]['CDGROPERREEX'];

			$operator = empty($row['EMAIL']) ? $this->environment->getCdOperador() : $this->findCdOperator($row['EMAIL']);

			if(!empty($row['EMAIL'])){
				$senha = $row['SENHA'];
				$this->validatePassword($operator, $row['SENHA']);
			}

			$dataSourceOperator = 'pla00900_auth_operator';
			$conditions = array(
				array(
					'columnName' => 'P_CDGRUPOPER',
					'operator' => '=',
					'value' => $cdGrupOperator
				),
				array(
					'columnName' => 'P_CDOPERADOR',
					'operator' => '=',
					'value' => $operator
				)
			);
			
			$auth = $this->crudService->getFilteredDataSet($dataSourceOperator, $conditions);

			if (empty($auth->getRows())) {
	      throw new \Exception("Operador não autorizado", 1);
	    }else {
	    	$response->addDataSet($auth);
	    }
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}
	
	public function findNrrequextr(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_DTINICIAL','P_DTFINAL', 'NRORG');
		$this->dataSourceName = 'pla00900_prevreti';
		parent::find($request, $response);
	}

	public function findPrevretiPrato(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI', 'P_NRREQUEXTR','P_DTPREVRETI','P_IDPREVRETI');
		$this->dataSourceName = 'prevreti_prato';
		parent::find($request, $response);
	}
	
	public function existData(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI', 'P_DTCARDAPIO', 'P_NRREQUEXTR');
		$this->dataSourceName = 'pla00900_exist_data';
		parent::find($request, $response);
	}
	
	public function horaServico(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI');
		$this->dataSourceName = 'pla00900_hora_servico';
		parent::find($request, $response);
	}
	
	public function nrprprevreti(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI', 'P_DTPREVRETI', 'P_IDPREVRETI', 'P_NRREQUEXTR');
		$this->dataSourceName = 'pla00900_nrprprevreti';
		parent::find($request, $response);
	}
	
	public function consumptionPerService(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI', 'P_IDTIPOCONS', 'P_DTCONSSERV');
		$this->dataSourceName = 'pla00900_consumption_per_service';
		parent::find($request, $response);
	}
	
	public function ferifili(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_DTFERIFILI');
		$this->dataSourceName = 'pla00900_ferifili';
		parent::find($request, $response);
	}

	public function findPrevretiPratoProduto(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDPRATO','P_CDFILIAL','P_CDSERVFILI', 'P_IDPREVRETI','P_CDFILAUXPRAT','P_CDTIPOPRAT', 'P_DTPREVRETI', 'P_NRREQUEXTR');
		$this->dataSourceName = 'prevreti_prato_produto';
		parent::find($request, $response);
	}

	public function findPrevretiProduto(DTO\Request\Filter $request, DTO\Response $response) {
		$this->parameterBagColumns = array('P_CDFILIAL','P_CDSERVFILI','P_DTPREVRETI','P_IDPREVRETI');
		$this->dataSourceName = 'pla00900_prevreti_produto';
		parent::find($request, $response);
	}
	
    public function retornaServicos($cdfilial, $nrorg) {
        $sql = <<<SQL
            SELECT LISTAGG(CDSERVFILI, '_') WITHIN GROUP (ORDER BY CDSERVFILI) CDLSERVFILI
            FROM SERVICO
            WHERE CDFILIAL = :CDFILIAL
            AND NRORG = :NRORG
SQL;
        $params = array(
            'NRORG'        => $nrorg,
            'CDFILIAL'     =>$cdfilial
        );

        try {
            $lista = $this->entityManager->getConnection()->fetchAll($sql, $params);
            return $lista[0]['CDLSERVFILI'];
        } catch (\Exception $e) {
            throw $e;
        }
    }
	
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $this->qr2Strategy->setQr2Type($row['QR2TYPE']);
        
        $cdfilial    = $row['CDFILIAL'];
        $nrorg       = $row['NRORG'];
		//coleta de dados
		$this->libUtilities->saveCollectedData($cdfilial,'REQUISICAO');
        if ($row['CDSERVFILI'] == 'T') {
            $row['CDSERVFILI'] = $this->retornaServicos($cdfilial, $nrorg);
        }
    }
    
    public function paramServ(DTO\Request $request, DTO\Response $response) {
		$sql = "SELECT
				 SE.IDUTRETIPRAT,
				 PP.CDTPRCONSFIX,
				 PP.CDPRACONSFIX,
				 PP.CDFILCONSFIX,
				 PP.CDTPRCONSPER,
				 PP.CDPRACONSPER,
				 PP.CDFILCONSPER
				FROM
				 PARAMPLANEJA PP,
				 SERVICO SE
				WHERE
				 PP.CDFILIAL = SE.CDFILIAL AND
				 SE.CDSERVFILI = :P_CDSERVFILI AND
				 PP.CDFILIAL = :P_CDFILIAL";
		
		$filter = $request->getParameters()['filter'];
		
		$params = array(
			"P_CDFILIAL" => $filter[0]['value'],
			"P_CDSERVFILI" => $filter[1]['value']
		);
		$result = $this->connection->fetchAll($sql, $params);
 	 
 		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('pla00900_paramserv', $result));
 	}
    
    public function verifyProducts(DTO\Request $request, DTO\Response $response) {
		$sql =	 "SELECT
					:P_CDOPERADOR as CDOPERADOR,
					CS.CDFILIAL as CDFILIAL,
					CS.CDSERVFILI as CDSERVFILI,
					CS.DTCONSSERV as DTCONSSERV,
					:P_NRREQUEXTR as NRREQUEXTR,
					CS.IDQTDECONS as IDQTDECONS,
					CS.CDPRODUTO as CDPRODUTO,
					TO_DATE(:P_DTLIMITERETI) as DTLIMITERETI,
					:P_HRLIMITERETI as HRLIMITERETI,
					CS.QTCONSSERV as QTCONSSERV,
					:P_CDSETORCOZ as CDSETORCOZ,
					:P_IDPREVTPPRAT as IDPREVTPPRAT,
					:P_NRPRPREVRETI as NRPRPREVRETI
					FROM
					CONSSERV CS
					WHERE
					(CS.CDFILIAL = :P_CDFILIAL) AND
					(CS.CDSERVFILI = :P_CDSERVFILI) AND
					(CS.IDTIPOCONS = :P_IDTIPOCONS) AND
					(CS.IDQTDECONS = :P_IDQTDECONS) AND
					(INSTR(:P_LISTA, CS.CDPRODUTO) > 0 ) AND
					(CS.DTCONSSERV >= (SELECT
					MAX(DTCONSSERV) DTCONSSERV
					FROM
					CONSSERV
					WHERE
					DTCONSSERV <= :P_DTCONSSERV AND
					CDFILIAL = :P_CDFILIAL AND
					CDSERVFILI = :P_CDSERVFILI)
					) AND
					NOT EXISTS(SELECT
					PR.CDPRODUTO
					FROM
					PREVRETI PR
					WHERE
					(PR.CDFILIAL = :P_CDFILIAL) AND
					(PR.CDSERVFILI = :P_CDSERVFILI) AND
					(PR.DTPREVRETI = :P_DTCONSSERV) AND
					(PR.NRREQUEXTR = :P_NRREQUEXTR) AND
					(PR.IDPREVRETI = :P_IDQTDECONS) AND
					(PR.CDPRODUTO = CS.CDPRODUTO) AND
					(INSTR(:P_LISTA, CS.CDPRODUTO) > 0 )
					)
					ORDER BY
					CS.CDFILIAL,
					CS.CDSERVFILI,
					CS.IDQTDECONS,
					CS.IDTIPOCONS,
					CS.DTCONSSERV";
		$row = $request->getRow();
		$params = array(
			"P_NRREQUEXTR" => $row['NRREQUEXTR'],
			"P_CDFILIAL" => $row['CDFILIAL'],
			"P_CDSERVFILI" => $row['CDSERVFILI'],
			"P_IDTIPOCONS" => $row['IDTIPOCONS'],
			"P_IDQTDECONS" => $row['IDQTDECONS'],
			"P_LISTA" => $row['LISTA'],
			"P_DTCONSSERV" => $row['DTPREVRETI'],
			"P_CDOPERADOR" => $row['CDOPERADOR'],
			"P_CDSETORCOZ" => $row['CDSETORCOZ'],
			"P_DTLIMITERETI"=> $row['DTLIMITERETI'],
			"P_HRLIMITERETI"=> $row['HRLIMITERETI'],
			"P_IDPREVTPPRAT"=> $row['IDPREVTPPRAT'],
			"P_NRPRPREVRETI"=> $row['NRPRPREVRETI'],
		);
		
		$result = $this->connection->fetchAll($sql, $params);
 		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('pla00900_verify_products', $result));
 	}
	
	public function findCdOperator($email) {
		$dataSourceName = 'operador';
		$conditions = array(
			array(
				'columnName' => 'DSEMAILOPER',
				'operator' => 'LIKE',
				'value' => $email
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(empty($dataSet)){
			throw new \Exception("Operador não cadastrado", 1);
		} else {
			return $dataSet[0]['CDOPERADOR'];
		}
	}
	
	public function validatePassword($operator, $password) {
		$dataSourceName = 'operador';
		$conditions = array(
			array(
				'columnName' => 'CDOPERADOR',
				'operator' => '=',
				"value" => $operator
			)
		);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions)->getRows();
		if(md5($password) != $dataSet[0]['CDSENHAOPERWEB']){
			throw new \Exception("Senha inválida", 1);
		}
	}

	public function autoOperation(DTO\Request $request, DTO\Response $response) {
		$row=$request->getRow();
		if(empty($row['P_NRREQUEXTR'])) {
			$row['P_NRREQUEXTR'] = $this->newCode->getCodeByTableName('REQUEXTR'.$row['P_CDFILIAL'], $this->environment->getNrOrgTrab(), 1, 10, false);
		}
		try {
			
			$procedure = new StoredProcedure($this->connection, 'CONSUMO_REQUISICAO_EXTRA');

			$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row['P_CDFILIAL'], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, $row['P_CDSERVFILI'], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DTCARDAPIO', Param::PARAM_INPUT, $row['P_DTCARDAPIO'], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRREQUEXTR', Param::PARAM_INPUT, $row['P_NRREQUEXTR'], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRCOMENSAIS', Param::PARAM_INPUT, $row['P_NRCOMENSAIS'], Param::PARAM_TYPE_INT));
			$procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDTIPOCONS', Param::PARAM_INPUT, $row['P_IDTIPOCONS'], Param::PARAM_TYPE_STR));

			$procedure->execute();
			$dataSet = array(
			    'P_NRREQUEXTR' => $row['P_NRREQUEXTR'],
			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('data', $dataSet));
		} catch (\Exception $e){
			$response->setError(new Error($e->getMessage(), 1));
		}
	}

	public function findRequextr(DTO\Request\Filter $request, DTO\Response $response) {
			$this->dataSourceName = 'pla00900_nrrequextr';
			$this->parameterBagColumns = array('CDFILIAL','CDSERVFILI','DTPREVRETI');
			parent::find($request, $response);
	}

	public function delete(DTO\Request $request, DTO\Response $response) {
		
		$rows = $request->getDataSet()->getRows();
		foreach ($rows as $row) {
			if(empty($row['CDPRODUTO'])){
				$this->service->deletePrevreti($row);
			} else {
				$this->dataSourceName = 'prevreti';
				parent::delete($request, $response);
			}
		}
	}

	public function savePratoOrProduto(DTO\Request $request, DTO\Response $response) {
		try {
			$params = $request->getRow();
			//coleta de dados
			$this->libUtilities->saveCollectedData($params['CDFILIAL'],'REQUISICAO');
			$rows = $params['produtos'];
			if(isset($params['CDSETORCOZ'])){
				$cdsetor =  $params['CDSETORCOZ'];
				$nmsetor = $params['NMSETORCOZ'];
			}else if (isset($rows[0]['CDSETORCOZ'])){
				$cdsetor = $rows[0]['CDSETORCOZ'];
				$nmsetor = $rows[0]['NMSETORCOZ'];
				$params['CDSETORCOZ'] = $cdsetor;
				$params['NMSETORCOZ'] = $nmsetor;
			}else{
				$cdsetor = null;
				$nmsetor = '';
			}
			$params['CDSETORCOZ'] = $cdsetor;
			$params['NMSETORCOZ'] = $nmsetor;
			$nrreqextr;
			if(empty($params['NRREQUEXTR'])) {
				$nrreqextr = $this->newCode->getCodeByTableName('REQUEXTR'.$params['CDFILIAL'], $this->environment->getNrOrgTrab(), 1, 10, false);
				$params['NRREQUEXTR'] = $nrreqextr;
			} else {
				$nrreqextr = $params['NRREQUEXTR'];
			}
			$erroToSave = array(
				'PRODUTOS' => array()
			);
			foreach ($rows as $row) {
				try {
					if((isset($row['__is_new']) && $row['__is_new'] == true) || (isset($params['__is_new']) && $params['__is_new'] == true)){
					     $row['CDFILIAL']  = $params['CDFILIAL'];
					     $row['CDSERVFILI']= $params['CDSERVFILI'];
					     $row['CDTIPOPRAT']= isset($params['CDTIPOPRAT']) ? $params['CDTIPOPRAT']: null;
					     $row['CDPRATO']   = isset($params['CDPRATO']) ? $params['CDPRATO'] : null;
					     $row['NRREQUEXTR']= $nrreqextr;
					     $row['DTPREVRETI']= $params['DTPREVRETI'];
					     $row['IDPREVRETI']= $params['IDPREVRETI'];
					     $row['CDSETORCOZ']= $cdsetor;
						 $valid = $this->service->verifyExisit($row, $row['CDPRODUTO']);
						if(!empty($valid)){
							throw new \Exception();
						}
					}
					$nrprevreti = self::newNrprprevreti($row, 1);
					$row['NRPRPREVRETI'] = str_pad($nrprevreti, 4, '0000', STR_PAD_LEFT);
					$this->dataSourceName = 'prevreti';
					(isset($row['__is_new']) && $row['__is_new'] == true) || (isset($params['__is_new']) && $params['__is_new'] == true) ? $this->service->insertPrevreti($row, $nrreqextr) : $this->service->updatePrevreti($row);
				
				} catch (\Exception $e) {
					array_push($erroToSave['PRODUTOS'], $row);
				}
			}
			$this->service->updateSetor($params);
			$dataSet = array(
			    'NRREQUEXTR' => $nrreqextr,
			    'CDSETORCOZ' => $cdsetor,
			    'NMSETORCOZ' => $nmsetor,
			    'ERROR'      => $erroToSave
			);
			$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('data', $dataSet));
		} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
    public function importConsumption(DTO\Request $request, DTO\Response $response){
        
        $rows = $request->getDataSet()->getRows();
        $count = 1;
        $nrReqExtr = $this->newCode->getCodeByTableName('REQUEXTR'.$rows[0]['CDFILIAL'], $this->environment->getNrOrgTrab(), 1, 10, false);
        foreach ($rows as $row) {
        	$nrprprevreti = is_null($row["NRPRPREVRETI"]) ? '0001' : $row["NRPRPREVRETI"];
        	$nrprprevreti = intval($nrprprevreti) + $count;
        	$nrprprevreti = str_pad(($nrprprevreti), 4, '0000', STR_PAD_LEFT);
            $params = array(
                ":P_CDOPERADOR" => $row["CDOPERADOR"],
                ":P_CDFILIAL" => $row["CDFILIAL"],
                ":P_CDSERVFILI" => $row["CDSERVFILI"],
                ":P_DTPREVRETI" => $row["DTPREVRETI"],
                ":P_NRREQUEXTR" => !empty($row["NRREQUEXTR"]) ? $row["NRREQUEXTR"] : $nrReqExtr,
                ":P_IDPREVRETI" => $row["IDPREVRETI"],
                ":P_CDPRODUTO" => $row["CDPRODUTO"],
                ":P_DTLIMITERETI" => $row["DTLIMITERETI"],
                ":P_HRLIMITERETI" => $row["HRLIMITERETI"],
                ":P_QTPREVRETI" => $row["QTPREVRETI"],
                ":P_CDSETORCOZ" => $row["CDSETORCOZ"],
                ":P_IDPREVTPPRAT" => $row["IDPREVTPPRAT"],
                ":P_NRPRPREVRETI" => $nrprprevreti ,
                ":P_CDTIPOPRAT" => $row["CDTIPOPRAT"],
                ":P_CDPRATO" => $row["CDPRATO"],
                ":P_CDFILAUXPRAT" => $row["CDFILAUXPRAT"]
            );
            $count++;
            
            
            
            $this->service->importConsumption($params);
        }
        $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('NRREQUEXTR', array($nrReqExtr)));

    }
	
	public function getMaxNvlProduto(DTO\Request $request, DTO\Response $response){
		
		$maxNvl = substr_count($this->service->getMaxNvlProduto(), '.')+1;
		$response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('CDPICTPROD', array($maxNvl)));
		
	}
	
	public function saveTipoDePratoEProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            //coleta de dados
			$this->libUtilities->saveCollectedData($dataSet[0]['CDFILIAL'],'REQUISICAO');
            if(!empty($dataSet)){
	            if(empty($dataSet[0]['NRREQUEXTR'])) {
					$nrrequextr = $this->newCode->getCodeByTableName('REQUEXTR'.$dataSet[0]['CDFILIAL'], $this->environment->getNrOrgTrab(), 1, 10, false);
				}else{
					$nrrequextr = $dataSet[0]['NRREQUEXTR'];
				}
	            $dataSet = self::verifyIsNewAndSetParams($dataSet, $nrrequextr);
	            
	            if($dataSet == "error"){
	            	throw new \Exception("Requisição extra já cadastrada!");
	            }
				
	            $this->service->updateSetor($dataSet[0]);
	            $dataSetPrato = new DataSet('pla00900_saveTipoPrato_Prod', $dataSet);
	            $dataSetPrato = new DTO\Request\DataSet($dataSetPrato, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
	            $this->dataSourceName = 'pla00900_saveTipoPrato_Prod';
	            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('data', array('NRREQUEXTR' => $nrrequextr, 'CDSETORCOZ' => $dataSet[0]['CDSETORCOZ'], 'NMSETORCOZ' => $dataSet[0]['NMSETORCOZ'])));
	            
	            self::save($dataSetPrato, $response);
            	
            }else throw new \Exception("Erro ao salvar!");
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function verifyIsNewAndSetParams($dataSet, $nrrequextr){
        $sql = "SELECT * FROM PREVRETI WHERE CDFILIAL = :CDFILIAL AND CDPRODUTO = :CDPRODUTO AND DTLIMITERETI = :DTLIMITERETI AND NRPRPREVRETI = :NRPRPREVRETI AND NRREQUEXTR = :NRREQUEXTR AND CDTIPOPRAT = :CDTIPOPRAT";
        $count = 0;
        foreach ($dataSet as $key => &$data) {
			$count++;
        	$nrprevreti = self::newNrprprevreti($dataSet[$key], $count);
			$dataSet[$key]['NRPRPREVRETI'] = str_pad(($nrprevreti), 4, '0000', STR_PAD_LEFT);
			$dataSet[$key]['NRREQUEXTR'] = $nrrequextr;
			$dataSet[$key]['IDPREVTPPRAT'] = 'S';
			
            $params = array("CDPRODUTO" => $dataSet[$key]['CDPRODUTO'],
            				"CDTIPOPRAT" => $dataSet[$key]['CDTIPOPRAT'],
            				"DTLIMITERETI" => $dataSet[$key]['DTLIMITERETI'],
            				"NRPRPREVRETI" => $dataSet[$key]['NRPRPREVRETI'],
            				"NRREQUEXTR" => $nrrequextr,
                            "CDFILIAL" => $dataSet[$key]['CDFILIAL']);
		    $query = $this->entityManager->getConnection()->prepare($sql);
            $query->execute($params);
            $result = $query->fetchAll();
            if(empty($result)){
                $dataSet[$key]['__is_new'] = true;
	            $valid = $this->service->verifyExisit($dataSet[$key], null);
				if(!empty($valid)){
					return "error";
				}
            }else{
                $dataSet[$key]['__is_new'] = false;
            }
        }
        return $dataSet;
    }
    
    public function newNrprprevreti ($row, $count){
    	$dataSourceName = 'pla00900_nvlnrprprevreti';
    	$conditions = array(
				array(
					'columnName' => 'P_CDSERVFILI',
					'operator' => '=',
					'value' => $row['CDSERVFILI']
				),
				array(
					'columnName' => 'P_CDFILIAL',
					'operator' => '=',
					'value' => $row['CDFILIAL']
				),
				array(
					'columnName' => 'P_NRREQUEXTR',
					'operator' => '=',
					'value' => $row['NRREQUEXTR']
				),
				array(
					'columnName' => 'P_DTPREVRETI',
					'operator' => '=',
					'value' => $row['DTPREVRETI']
				),
				array(
					'columnName' => 'P_IDPREVRETI',
					'operator' => '=',
					'value' => $row['IDPREVRETI']
				),
				array(
					'columnName' => 'P_NRORG',
					'operator' => '=',
					'value' => $this->environment->getNrOrgTrab()
				)
			);
        
        if(empty($row['NRPRPREVRETI'])) {
			$filterCriteria = new FilterCriteria($dataSourceName, $conditions);
			$this->parameterBagColumns = array('P_CDSERVFILI','P_CDFILIAL','P_NRREQUEXTR','P_DTPREVRETI','P_IDPREVRETI', 'P_NRORG');
			$filterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
			$dataSetNr = $this->dataSourceManager->findBy($filterCriteria)->getRows();
			$nrprevreti = (intval($dataSetNr[0]['NRPRPREVRETI'])) + $count;
			return $nrprevreti = strval($nrprevreti);
        }else{
        	return $nrprevreti = $row['NRPRPREVRETI'];
        }
    }
    
    public function removeTipoDePratoEProduto(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $dataSet = $this->service->getRowsToDelete($dataSet);
           
            foreach($dataSet as $key => &$data){
            	$dataSet[$key]["__is_new"] = false; 
            }
            
            $dataSetPrato = new DataSet('pla00900_deleteTipoPrato_Prod', $dataSet);
            $dataSetPrato = new DTO\Request\DataSet($dataSetPrato, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
            $this->dataSourceName = 'pla00900_saveTipoPrato_Prod';
            parent::delete($dataSetPrato, $response);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function updateTipoDePrato(DTO\Request $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $valid = $this->service->verifyExisit($row, null);
            
            if(empty($valid)){
            	$acceptFields = false;
            }else{
            	$acceptFields = self::validUpdate($row);
            }
            
            if(!empty($valid) && $acceptFields){
				throw new \Exception("Requisição extra já cadastrada!");
			}
			//coleta de dados
			$this->libUtilities->saveCollectedData($row['CDFILIAL'],'REQUISICAO');
			
            $this->service->updateTipoDePrato($row);
            $this->service->updateSetor($row);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('data', array('NRREQUEXTR' => $row['NRREQUEXTR'], 'CDSETORCOZ' => $row['CDSETORCOZ'], 'NMSETORCOZ' => $row['NMSETORCOZ'])));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function validUpdate($row){
    	$cdproduto = empty($row['CDPRODUTO']) ? '' : $row['CDPRODUTO'];
    	$cdprato = empty($row['CDPRATO']) ? '' : $row['CDPRATO'];
    	$cdtipoprato = empty($row['CDTIPOPRAT']) ? '' : $row['CDTIPOPRAT'];
    	$old_cdproduto = empty($row['OLD_CDPRODUTO']) ? '' : $row['OLD_CDPRODUTO'];
    	$old_cdprato = empty($row['OLD_CDPRATO']) ? '' : $row['OLD_CDPRATO'];
    	$old_cdtipoprato = empty($row['OLD_CDTIPOPRAT']) ? '' : $row['OLD_CDTIPOPRAT'];
    	
    	if($cdproduto == $old_cdproduto && $cdtipoprato == $old_cdtipoprato && $cdprato == $old_cdprato){
    		return false;
    	}else{
    		return true;
    	}
    }
    
    public function verifyRequest(DTO\Request $request, DTO\Response $response){
        try {
            $dataSet = $request->getDataSet()->getRows();
           
            $row = array();
            
            foreach ($dataSet as $key => &$data) {
            	$row[$data['name']] = $data['value'];
            }
            $dataSet = $this->service->verifyRequest($row);
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('data', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function exportRequest(DTO\Request\Row $request, DTO\Response $response){
    	try {
    		$row = $request->getRow();
    		$servicos = $row['ALL_SERVICE'] == 'S' ? $this->getServicos($row, $response) : $row['CDSERVFILIDESTINO'];
    		if($servicos != null) {
    			if($row['NRREQUEXTR'] == null){
    				$nrreqextr = $this->newCode->getCodeByTableName('REQUEXTR'.$row['CDFILIALDESTINO'], $this->environment->getNrOrgTrab(), 1, 10, false);
    				$row['NRREQUEXTR'] = $nrreqextr;
    			}
				$pratosProdutosOrigem = $this->getPratosProdutosOrigem($row);
				if($pratosProdutosOrigem != null) {
		    		$CDFILIAL_ORIGEM = $row['CDFILIALORIGEM'];
		    		$CDFILIAL_DESTINO = $row['CDFILIALDESTINO'];
		    		$IDUTRETIPRAT_ORIGEM = $row['IDUTRETIPRAT'];
		    		$nomeSetorCoz = strlen($row['NMSETORCOZ']) > 0 ? explode(" ", $row['NMSETORCOZ'])[2] : "";
		    		$selectDate = explode(" ", $row['DTPREVRETI'])[0];
		    		$countExport = 0;
		    		$errors = array(
		    			'NOT_MAPPED_ERROR' => array()
		    		);
		    		$errorPeriodo = $this->validatePeriodo($row['CDFILIALDESTINO'], $selectDate);
		    		if($errorPeriodo !== null){
		    			$errors = array_merge($errors, $errorPeriodo);
		    		} else{
			    		foreach ($servicos as $currentService){
			    			if($currentService['IDUTRETIPRAT'] === $IDUTRETIPRAT_ORIGEM){
			    				$nrreqextr = $this->newCode->getCodeByTableName('REQUEXTR'.$CDFILIAL_DESTINO, $this->environment->getNrOrgTrab(), 1, 10, false);
				    			foreach($pratosProdutosOrigem as &$current){
				    				try {
						    			$current['NRREQUEXTR'] = $nrreqextr;
						    			$current['__is_new'] = true;
						    			$current['CDSERVFILI'] = $currentService['CDSERVFILI'];
						    			$current['CDFILIAL'] = $CDFILIAL_DESTINO;
									    $current['DTPREVRETI'] = $row['DTPREVRETI'];
									    $current['IDPREVRETI'] = $row['IDPREVRETI'];
									    $nrprevreti = self::newNrprprevreti($current, 1);
									    $current['NRPRPREVRETI'] = str_pad($nrprevreti, 4, '0000', STR_PAD_LEFT);
									    $current['NMSETORCOZ'] = $nomeSetorCoz;
									    if($current['CDSETORCOZ']) {
									    	$this->service->insertSetorcoz($current);
									    }
									    $this->service->insertPrevreti($current, $nrreqextr);
									    $countExport++;
								    } catch (\Exception $e){
								    	if(strpos($e->getMessage(), 'ORA-00001')){
								    		if(!isset($errors['SERVICO_CADASTRADO'])) $errors['SERVICO_CADASTRADO'] = array();
								    		array_push($errors['SERVICO_CADASTRADO'], $currentService);
								    	} else {
								    		$currentService['ERROR'] = $e->getMessage();
								    		array_push($errors['NOT_MAPPED_ERROR'], $currentService);
								    	}
								    }
				    			}	
			    			} else {
			    				if(!isset($errors['DIFERENTE_TIPO_RETIRADA'])) $errors['DIFERENTE_TIPO_RETIRADA'] = array();
			    					array_push($errors['DIFERENTE_TIPO_RETIRADA'], $currentService);
			    			}
			    		}
		    		}
		    		$response->addDataSet(new DataSet("ERRORS", $errors));
		    		$response->addDataSet(new DataSet("SUCCESS_EXPORT", array('COUNT' => $countExport)));
    			} else {
    				$errors = array(
	    				'NOT_MAPPED_ERROR' => array()
	    			);
	    			if(!isset($errors['SEM_PRODUTO_EXPORTACAO'])) $errors['SEM_PRODUTO_EXPORTACAO'] = array();
	    			$response->addDataSet(new DataSet("ERRORS", $errors));
    			}
    		} else {
    			$errors = array(
	    			'NOT_MAPPED_ERROR' => array()
	    		);
	    		if(!isset($errors['servicoNaoFiltrado'])) $errors['servicoNaoFiltrado'] = array();
	    		$response->addDataSet(new DataSet("ERRORS", $errors));
    		}
    	} catch (\Exception $e){
			$response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString())); 		
    	}
    }
    
    private function validatePeriodo($cdfilial, $selectDate){
    	$dataProc   = $this->getDataProcessamento($cdfilial);
    	$dataProc   = $this->dateUtil->getDataDeString($dataProc, 'd/m/Y', true);
    	$selectDate = $this->dateUtil->getDataDeString($selectDate, 'd/m/Y', true);
    	$minDate = $this->dateUtil->subtraiIntervalo($dataProc, 10);
		$maxDate = $this->dateUtil->adicionaIntervalo($dataProc, 10);
		if($this->dateUtil->comparaDatas($minDate, $selectDate) && $this->dateUtil->comparaDatas($selectDate, $maxDate)){
			return null;
		} else {
			return array( 'dateMustBeWithinRange' => array(
				0 => array(
					'DATAINICIAL' => $minDate->format('d/m/Y'),
					'DATAFINAL'   => $maxDate->format('d/m/Y')
				)
			));
		}
    }
    
    private function getPratosProdutosOrigem($row){
    	$conditions = array(
			array(
				'columnName' => 'NRREQUEXTR',
				'operator'  => '=',
				'value'    => $row['NRREQUEXTR']
			),
			array(
				'columnName' => 'CDFILIAL',
				'operator'  => '=',
				'value'    => $row['CDFILIALORIGEM']
			),
			array(
				'columnName' => 'CDSERVFILI',
				'operator'  => '=',
				'value'    => $row['CDSERVFILIORIGEM']
			),
			array(
				'columnName' => 'DTPREVRETI',
				'operator'  => '=',
				'value'    => $row['DTPREVRETI']
			),
			array(
				'columnName' => 'IDPREVRETI',
				'operator'  => '=',
				'value'    => $row['IDPREVRETI']
			),
			array(
				'columnName' => 'NRORG',
				'operator' => '=',
				'value' => $this->environment->getNrOrgTrab()
			)
    	);
    	$filterCriteria = new FilterCriteria("pla00900_saveTipoPrato_Prod", $conditions);
		$dataSet = $this->dataSourceManager->findBy($filterCriteria)->getRows();
		return $dataSet;
    }
    
    private function getServicos($row){
    	$sql = "SELECT CDFILIAL, CDSERVFILI, NMSERVICO, HRINICATEN, IDUTRETIPRAT, IDBLOQALTPRA, NRORG FROM ( SELECT SO.CDFILIAL, SO.CDSERVFILI, SE.NMSERVICO, SE.HRINICATEN, SE.IDUTRETIPRAT, SE.IDBLOQALTPRA, SE.NRORG FROM SERVOPER   SO, SERVICO    SE WHERE ( SO.CDFILIAL = :CDFILIAL ) AND ( SO.CDOPERADOR = :CDOPERADOR ) AND ( SE.CDFILIAL = :CDFILIAL ) AND ( SE.CDSERVFILI = SO.CDSERVFILI ) AND ( ( SE.IDSERVATIV <> 'N' ) OR ( SE.IDSERVATIV IS NULL ) ) AND ( SE.IDTIPOSERV <> 'A' ) UNION ALL SELECT CDFILIAL, CDSERVFILI, NMSERVICO, HRINICATEN, IDUTRETIPRAT, IDBLOQALTPRA, NRORG FROM SERVICO WHERE ( CDFILIAL = :CDFILIAL ) AND ( IDTIPOSERV <> 'A' ) AND ( ( IDSERVATIV <> 'N' ) OR ( IDSERVATIV IS NULL ) ) AND NOT EXISTS ( SELECT 1 FROM SERVOPER   SO, SERVICO    SE WHERE ( SO.CDFILIAL = :CDFILIAL ) AND ( SO.CDOPERADOR = :CDOPERADOR ) AND ( SE.CDFILIAL = :CDFILIAL ) AND ( SE.CDSERVFILI = SO.CDSERVFILI ) AND ( ( SE.IDSERVATIV <> 'N' ) OR ( SE.IDSERVATIV IS NULL ) ) AND ( SE.IDTIPOSERV <> 'A' ) ) ) WHERE CDSERVFILI <> :CDSERVFILI ORDER BY CDFILIAL, NLSSORT(CDSERVFILI, 'NLS_SORT=BINARY_AI')";
		$params = array(
			"CDFILIAL"   => $row['CDFILIALDESTINO'],
        	"CDSERVFILI" => $row['CDSERVFILIORIGEM'],
        	"CDOPERADOR" => $this->environment->getCdOperador()
        );
	    $query = $this->entityManager->getConnection()->prepare($sql);
	    
        $query->execute($params);
        return $query->fetchAll();
    }
    
    private function getDataProcessamento($CDFILIAL){
    	//$sql = "SELECT IDUTILDTPROC FROM PARAMGERAL";
    	//$query = $this->entityManager->getConnection()->prepare($sql);
		//$query->execute();
    	//$utilizaDataProcessamento = $query->fetch()['IDUTILDTPROC'];
    	
    	//if($utilizaDataProcessamento == 'N') return date('d/m/Y');
    	
    	$sql = "SELECT CASE WHEN NVL(IDUTILDTPROC,'N') = 'N' THEN TO_CHAR(SYSDATE, 'DD/MM/YYYY') ELSE TO_CHAR(NVL(DTPROCESSA, SYSDATE), 'DD/MM/YYYY') END DTPROCESSA FROM PARAMFILIAL PF, PARAMGERAL WHERE PF.CDFILIAL = :CDFILIAL";
		$params = array(
			"CDFILIAL"   => $CDFILIAL
        );
	    $query = $this->entityManager->getConnection()->prepare($sql);
	    
        $query->execute($params);
        return $query->fetch()['DTPROCESSA'];
    	
    }
}